# Wait time for invalid input message
MSG_WAIT_TIME = 0.3
DEFAULT_ON_INTERRUPT = "raise"
DEFAULT_ON_BLANK = "return"
DEFAULT_FRONTEND = "auto"
DEFAULT_ONCE = False
DEFAULT_RETURN_MODE = "flat"
DEFAULT_REMEMBER = True
DEFAULT_CLEAR = False
_CASE_TITLE = "__meny_title__"
_CASE_IGNORE = "__meny_ignore__"
_DICT_KEY = "__meny_key_from_input_dict__"
_ROOT = "__meny_root__"
