PURPLE = "\x1b[95m"
CYAN = "\x1b[96m"
DARKCYAN = "\x1b[36m"
BLUE = "\x1b[94m"
GREEN = "\x1b[92m"
YELLOW = "\x1b[93m"
RED = "\x1b[91m"
BOLD = "\x1b[1m"
UNDERLINE = "\x1b[4m"
END = "\x1b[0m"

START_MSG = "Starting console interface"
LOGO_TITLE = " Main menu "

ENTER_PROMPT = "Input"

EXIT_MSG = "Exiting program"
INVALID_TERMINAL_INPUT_MSG = "Invalid input"

INPUT_BLANK_HINT_RETURN = "Enter blank to return to parent menu"
INPUT_BLANK_HINT_PASS = ""
INPUT_BLANK_HINT_EXIT = "Enter blank to exit program"

INPUT_WAIT_PROMPT_MSG = "Press enter to continue"

DEFAULT_TITLE = " Title "
